﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.SearchModels;
using FishFactoryContracts.StoragesContracts;
using FishFactoryContracts.ViewModels;
using FishFactoryDatabaseImplement.Models;
using FishFactoryDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FishFactoryDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public List<ShopViewModel> GetFullList()
        {
            using var context = new FishFactoryDatabase();
            return context.Shops
                    .Include(x => x.ListCannedFk)
                    .ThenInclude(x => x.Canned)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            using var context = new FishFactoryDatabase();
            return context.Shops
                    .Include(x => x.ListCannedFk)
                    .ThenInclude(x => x.Canned)
                    .Where(x => x.ShopName.Contains(model.ShopName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new FishFactoryDatabase();
            return context.Shops
                .Include(x => x.ListCannedFk)
                .ThenInclude(x => x.Canned)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new FishFactoryDatabase();
            var newShop = Shop.Create(context, model);
            if (newShop == null)
            {
                return null;
            }
            context.Shops.Add(newShop);
            context.SaveChanges();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new FishFactoryDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.FirstOrDefault(rec => rec.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }
                shop.Update(model);
                context.SaveChanges();
                shop.UpdateSushi(context, model);
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new FishFactoryDatabase();
            var element = context.Shops
                .Include(x => x.ListCannedFk)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Shops.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
        public bool SellCanneds(ICannedModel model, int count)
        {
            using var context = new FishFactoryDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                foreach (var shop in context.Shops
                    .Include(x => x.ListCannedFk)
                    .ThenInclude(x => x.Canned)
                    .ToList()
                    .Where(x => x.ListCanneds.ContainsKey(model.Id)))
                {
                    int countInCurrentShop = shop.ListCanneds[model.Id].Item2;
                    if (countInCurrentShop <= count)
                    {
                        var elem = context.ShopCanned
                            .Where(x => x.CannedId == model.Id)
                            .FirstOrDefault(x => x.ShopId == shop.Id);
                        context.ShopCanned.Remove(elem);
                        shop.ListCanneds.Remove(model.Id);
                        count -= countInCurrentShop;
                    }
                    else
                    {
                        shop.ListCanneds[model.Id] = (shop.ListCanneds[model.Id].Item1, countInCurrentShop - count);
                        count = 0;
                        shop.UpdateSushi(context, new()
                        {
                            Id = shop.Id,
                            ListCanneds = shop.ListCanneds,
                        });
                    }
                    if (count == 0)
                    {
                        context.SaveChanges();
                        transaction.Commit();
                        return true;
                    }
                }
                transaction.Rollback();
                return false;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
    }
}
