﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.ViewModels;
using FishFactoryDataModels;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FishFactoryDataModels.Models;

namespace FishFactoryDatabaseImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        [Required]
        public string ShopName { get; private set; } = string.Empty;
        [Required]
        public string Address { get; private set; } = string.Empty;
        [Required]
        public DateTime DateOpening { get; private set; }
        [Required]
        public int MaxCountCanneds { get; private set; }

        private Dictionary<int, (ICannedModel, int)>? _shopCanned = null;

        [NotMapped]
        public Dictionary<int, (ICannedModel, int)> ListCanneds
        {
            get
            {
                if (_shopCanned == null)
                {
                    _shopCanned = ListCannedFk.ToDictionary(recPC => recPC.CannedId, recPC => (recPC.Canned as ICannedModel, recPC.Count));
                }
                return _shopCanned;
            }
        }

        [ForeignKey("ShopId")]
        public virtual List<ShopCanned> ListCannedFk { get; set; } = new();

        public static Shop Create(FishFactoryDatabase context, ShopBindingModel model)
        {
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpening = model.DateOpening,
                MaxCountCanneds = model.MaxCountCanneds,
                ListCannedFk = model.ListCanneds.Select(x => new ShopCanned
                {
                    Canned = context.Canneds.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }

        public void Update(ShopBindingModel model)
        {
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpening = model.DateOpening;
            MaxCountCanneds = model.MaxCountCanneds;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpening = DateOpening,
            MaxCountCanneds = MaxCountCanneds,
            ListCanneds = ListCanneds
        };

        public void UpdateSushi(FishFactoryDatabase context, ShopBindingModel model)
        {
            var shopCanned = context.ShopCanned.Where(rec => rec.ShopId == model.Id).ToList();
            if (shopCanned != null && shopCanned.Count > 0)
            {   // удалили те, которых нет в модели
                context.ShopCanned.RemoveRange(shopCanned.Where(rec => !model.ListCanneds.ContainsKey(rec.CannedId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updateCanned in shopCanned)
                {
                    updateCanned.Count = model.ListCanneds[updateCanned.CannedId].Item2;
                    model.ListCanneds.Remove(updateCanned.CannedId);
                }
                context.SaveChanges();
            }
            var shop = context.Shops.First(x => x.Id == Id);
            foreach (var ss in model.ListCanneds)
            {
                context.ShopCanned.Add(new ShopCanned
                {
                    Shop = shop,
                    Canned = context.Canneds.First(x => x.Id == ss.Key),
                    Count = ss.Value.Item2
                });
                context.SaveChanges();
            }
            _shopCanned = null;
        }
    }
}
